/*
 * Az oldal betöltése után futnak le a jQuery kódok.
 */
$(function (){
	var desc = 'Ez egy bemutató program, amely felhasználó adatokat tölt be az adatbázisból mező név - érték párosítással. '
		   + 'Az \'Újra tölt\' gombra kattintva újra be lehet tölteni az adatokat. '
		   + 'Az adatokat tartalmazó felületre kattintva el lehet távolítani azt.';
	
	/*
	 * Emuláljuk a betöltés egy 3 másodperces várakozással.
	 * Utána eltávolítjuk a betöltő felületet és fel csúsztatjuk a
	 * program főcímét.
	 */
	$('#loading-msg').delay(3000).fadeOut('slow', function (){		
		$(this).remove();
		$('#loading-mask').remove();
		//Felcsúsztatjuk és megjelenítünk egy gombot alá, valamint a leírást.
		$('#loading').animate({top: '0'}, 1500, function (){
			$(this).append($('<div>' + desc + '</div>'));
			var button = $('<input type="button" value="Újra tölt"/>');
			button.on('click', function () {
				load();
			});
			$(this).append(button);
			load();
		});
		
	});

	/*
	 * Az AJAX hívást megvalósító függvény.
	 * JSON-ben várjuk az eredményt, ha sikeresen megkaptuk,
	 * akkor feldolgozzuk az adatokat.
	 */
	var load = function (){
		$.ajax({
			url: '/data',
			type: 'GET',
			dataType: 'json',
			success: function (data){
				var docs = data.response.data;				
				for (var i in docs){
					var doc = docs[i].doc;
					console.log(docs[i].doc);
					var msg = createData(doc);
					viewer.add(msg);
				}
			},
			complete: function (req, status){
				console.log(status);
			}
		});
	};

        /*
	 * A dokumentumot, mint egy asszociatív tömböt feldolgozzuk
	 * és kiírjuk a kulcs: érték párokat.
 	 */
	var createData = function (doc){
		var msg = '';
		for (var d in doc){
			if (d != '_id' && d != '_rev'){
				msg += d + ': ';
				if (d === 'url')
				   msg += '<a href="';
				msg += doc[d];
				if (d === 'url')
				  msg += '">' + doc[d] + '</a>';
				msg +='<br/>';
			}
		}
		return msg;
	};

	/*
	 * Az adatokat megjelenítő modul, amely 
	 * listába rakja az adatokat, majd ha rákattintunk egy listaelemre,
	 * eltűnik.
	 */
	var viewer = (function () {
		var $ul = $('<ul id="adatok" />'),
		
		//inicializálás, lista létrehozása
		init = function () {
			$ul.prependTo('#main');
			$ul.on('click', 'li.noauto', function () {
				vege($(this));
			});
		},
		//hozzáadás és megjelenítés
		add = function (msg) {
			
			var $li = $('<li>'+msg+'</li>');
			$li.hide();
			$ul.append($li);
			$li.fadeIn('normal');
			$li.addClass('noauto');
			
			return $li;
		},
		//eltávolítás
		vege = function ($li) {
			$li.fadeOut('normal', function () {
				$li.remove();
			});
		};

		init();

		return {
			add: add,
			vege: vege
		};
	}());  
});
